#!/usr/bin/perl


#------------------------------------
# subroutine print_license_info
# print relevant license information 
#------------------------------------
sub print_license_info {
$license_info= <<END;

Copyright Syngenta Limited 2007
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

THIS PROGRAM IS MADE AVAILABLE FOR DISTRIBUTION WITHOUT ANY FORM OF WARRANTY TO THE 
EXTENT PERMITTED BY APPLICABLE LAW.  THE COPYRIGHT HOLDER PROVIDES THE PROGRAM \"AS IS\" 
WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT  
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
PURPOSE. THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM LIES
WITH THE USER.  SHOULD THE PROGRAM PROVE DEFECTIVE IN ANY WAY, THE USER ASSUMES THE
COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION. THE COPYRIGHT HOLDER IS NOT 
RESPONSIBLE FOR ANY AMENDMENT, MODIFICATION OR OTHER ENHANCEMENT MADE TO THE PROGRAM 
BY ANY USER WHO REDISTRIBUTES THE PROGRAM SO AMENDED, MODIFIED OR ENHANCED.

IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL THE 
COPYRIGHT HOLDER BE LIABLE TO ANY USER FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL,
INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE THE
PROGRAM (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE
OR LOSSES SUSTAINED BY THE USER OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO 
OPERATE WITH ANY OTHER PROGRAMS), EVEN IF SUCH HOLDER HAS BEEN ADVISED OF THE 
POSSIBILITY OF SUCH DAMAGES.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

END

print "$license_info\n";
}


# gromacs execution script
# deals with proteins only, solvated and with counter ions.
# usage is ./scriptname file.pdb 
# To adjust between test and production runs modify the
# number of steps at each EM and MD stage using the parameter nsteps



#------------------------------------
# subroutine run_command 
# runs and operating systems command 
#------------------------------------
sub run_command {
local ($command) = @_;
local ($dummy);
print "RUNNING $command\n";
system($command);
print "FINISHED $command\n";
}




#------------------------------------
# subroutine get_input
# to get an input after prompting the user 
# usage:	$name=&get_input("Enter name...");
#------------------------------------
sub get_input {
local ($prompt) = @_;
local $value;
print "$prompt";
$value=<STDIN>;
return($value);
}


#
#------------------------------------
# function to create molecular dynamics parameter file
# create input file for position restrained MD
# fixed filename of pr.mdp
#------------------------------------
#
sub create_pr_mdp_file {
local ($prefix,$charge) = @_;
open(FILE1,">pr.mdp") || die "cant open pr.mdp file: $! \n";
#print FILE1 "This is a test message \n";
#print FILE1 "$prefix\n";
#print FILE1 "$charge\n";
print FILE1 "title               =  $prefix ; position restraining\n";
print FILE1 "warnings            =  10\n";
print FILE1 "cpp                 =  /usr/bin/cpp\n";
print FILE1 "define              =  -DPOSRES\n";
print FILE1 "constraints         =  all-bonds\n";
print FILE1 "integrator          =  md\n";
print FILE1 "dt                  =  0.002    ; ps !\n";
print FILE1 "nsteps              =  1000     ; total 2.0 ps.\n";
print FILE1 "nstcomm             =  1\n";
print FILE1 "nstxout             =  500 ; collect data every 1.0 ps\n";
print FILE1 "nstvout             =  1000\n";
print FILE1 "nstfout             =  0\n";
print FILE1 "nstlog              =  10\n";
print FILE1 "nstenergy           =  10\n";
print FILE1 "nstlist             =  10\n";
print FILE1 "ns_type             =  grid\n";
print FILE1 "rlist               =  0.9\n";
print FILE1 "coulombtype         =  PME\n";
print FILE1 "rcoulomb            =  0.9\n";
print FILE1 "rvdw                =  0.9\n";
print FILE1 "fourierspacing      =  0.12\n";
print FILE1 "fourier_nx          =  0\n";
print FILE1 "fourier_ny          =  0\n";
print FILE1 "fourier_nz          =  0\n";
print FILE1 "pme_order           =  4\n";
print FILE1 "ewald_rtol          =  1e-5\n";
print FILE1 "optimize_fft        =  yes\n";
print FILE1 "; Berendsen temperature coupling is on in two groups\n";
print FILE1 "Tcoupl              =  berendsen\n";
if ($charge==0) {
print FILE1 "tau_t               =  0.1              0.1\n";
print FILE1 "tc-grps             =  protein          sol\n";
print FILE1 "ref_t               =  300              300\n";
}
if ($charge<0) {
print FILE1 "tau_t               =  0.1              0.1        0.1\n";
print FILE1 "tc-grps             =  protein          sol        NA+\n";
print FILE1 "ref_t               =  300              300        300\n";
}
if ($charge>0) {
print FILE1 "tau_t               =  0.1              0.1        0.1\n";
print FILE1 "tc-grps             =  protein          sol        CL-\n";
print FILE1 "ref_t               =  300              300        300\n";
}
print FILE1 "; Pressure coupling on\n";
print FILE1 "Pcoupl              =  berendsen\n";
print FILE1 "tau_p               =  0.5\n";
print FILE1 "compressibility     =  4.5e-5\n";
print FILE1 "ref_p               =  1.0\n";
print FILE1 "; Generate velocites is on at 300 K.\n";
print FILE1 "gen_vel             =  yes\n";
print FILE1 "gen_temp            =  300.0\n";
print FILE1 "gen_seed            =  173529\n";
close(FILE1);
}




#
#------------------------------------
# function to create molecular dynamics parameter file
# create input file for regular (non position restrained) MD
# fixed filename of md.mdp
#------------------------------------
#
sub create_md_mdp_file {
local ($prefix,$charge) = @_;
open(FILE1,">md.mdp") || die "cant open md.mdp file: $! \n";
#print FILE1 "This is a test message \n";
#print FILE1 "$prefix\n";
#print FILE1 "$charge\n";

print FILE1 "title               =  $prefix ; Regular (non pr MD)\n";
print FILE1 "cpp                 =  /usr/bin/cpp\n";
print FILE1 "constraints         =  all-bonds\n";
print FILE1 "integrator          =  md\n";
print FILE1 "dt                  =  0.002    ; ps !\n";
print FILE1 "nsteps              =  10000     ; total 20.0 ps.\n";
print FILE1 "nstcomm             =  1\n";
print FILE1 "nstxout             =  500 ; collect data every 1.0 ps\n";
print FILE1 "nstvout             =  0\n";
print FILE1 "nstfout             =  0\n";
print FILE1 "nstlist             =  10\n";
print FILE1 "ns_type             =  grid\n";
print FILE1 "rlist               =  0.9\n";
print FILE1 "coulombtype         =  PME\n";
print FILE1 "rcoulomb            =  0.9\n";
print FILE1 "rvdw                =  0.9\n";
print FILE1 "fourierspacing      =  0.12\n";
print FILE1 "fourier_nx          =  0\n";
print FILE1 "fourier_ny          =  0\n";
print FILE1 "fourier_nz          =  0\n";
print FILE1 "pme_order           =  4\n";
print FILE1 "ewald_rtol          =  1e-5\n";
print FILE1 "optimize_fft        =  yes\n";
print FILE1 "; Berendsen temperature coupling is on in two groups\n";
print FILE1 "Tcoupl              =  berendsen\n";
if ($charge==0) {
print FILE1 "tau_t               =  0.1              0.1\n";
print FILE1 "tc-grps             =  protein          sol\n";
print FILE1 "ref_t               =  300              300\n";
}
if ($charge<0) {
print FILE1 "tau_t               =  0.1              0.1          0.1\n";
print FILE1 "tc-grps             =  protein          sol          NA+\n";
print FILE1 "ref_t               =  300              300          300\n";
}
if ($charge>0) {
print FILE1 "tau_t               =  0.1              0.1          0.1\n";
print FILE1 "tc-grps             =  protein          sol          CL-\n";
print FILE1 "ref_t               =  300              300          300\n";
}
print FILE1 "; Pressure coupling on\n";
print FILE1 "Pcoupl              =  berendsen\n";
print FILE1 "tau_p               =  0.5\n";
print FILE1 "compressibility     =  4.5e-5\n";
print FILE1 "ref_p               =  1.0\n";
print FILE1 "; Generate velocites is on at 300 K.\n";
print FILE1 "gen_vel             =  yes\n";
print FILE1 "gen_temp            =  300.0\n";
print FILE1 "gen_seed            =  173529\n";
close(FILE1);
}


#
#------------------------------------
# function to create em.mdp file
# energy minimisation parameter file 
# based on J.E.Kerrigan tutorial
#------------------------------------
#
sub create_em_mdp_file {
local ($prefix) = @_;
open(FILE1,">em.mdp") || die "cant open em.mdp file: $! \n";
#print FILE1 "This is a test message \n";
#print FILE1 "$prefix\n";
#print FILE1 "$charge\n";
print FILE1 "title               =  $prefix\n";
print FILE1 "cpp                 =  /usr/bin/cpp\n";
print FILE1 "define              =  -DFLEX_SPC\n";
print FILE1 "constraints         =  none\n";
print FILE1 "integrator          =  steep\n";
print FILE1 "dt                  =  0.002    ; ps !\n";
print FILE1 "nsteps              =  5000\n";
print FILE1 "nstlist             =  10\n";
print FILE1 "ns_type             =  grid\n";
print FILE1 "rlist               =  0.9\n";
print FILE1 "coulombtype         =  PME\n";
print FILE1 "rcoulomb            =  0.9\n";
print FILE1 "rvdw                =  0.9\n";
print FILE1 "fourierspacing      =  0.12\n";
print FILE1 "fourier_nx          =  0\n";
print FILE1 "fourier_ny          =  0\n";
print FILE1 "fourier_nz          =  0\n";
print FILE1 "pme_order           =  4\n";
print FILE1 "ewald_rtol          =  1e-5\n";
print FILE1 "optimize_fft        =  yes\n";
print FILE1 ";\n";
print FILE1 ";       Energy minimizing stuff\n";
print FILE1 ";\n";
print FILE1 "emtol               =  1000.0\n";
print FILE1 "emstep              =  0.01\n";
close(FILE1);
}


#------------------------------------
# subroutine to create sol.txt file 
# no parameters 
#------------------------------------
sub create_sol_txt_file {
open(FILE1,">sol.txt") || die "cant open em.mdp file: $! \n";
print FILE1 "12";
close(FILE1);
}


#------------------------------------
# subroutine to modify Gromacs topology file
# reduce number of SOL molecules by number
# of ions added, this is system charge value,
# write line for CL- or NA+ ions.
#------------------------------------
sub modify_top_file {
local ($topfilename,$charge) = @_;
local ($oldfilename);
local ($line,$dummy);
local ($nsol,$nsolminus,$abcharge);
# find absolute value of charge
$abcharge=abs($charge);
# if no charge, then no ions to be added, so just return
if ($abcharge==0) {
        return;
}
# copy topology file to version prefixed with .old
$oldfilename="old.".$topfilename;
system("mv $topfilename $oldfilename");

# open copy of topology file, scan line by line and replace relevant items
open(FILE1,"$oldfilename") || die "cant open file $oldfilename: $! \n";
open(FILE2,">$topfilename") || die "cant open file $topfilename: $! \n";
while ($_ = <FILE1>) {
	$line=$_;
# if line contains string SOL then decrement number of solvent molecules
	if (substr($line,0,3) eq 'SOL') {
		($dummy,$nsol)=split(' ',$line);
		$nsolminus=$nsol-$abcharge;
		#print "nsol, abcharge, nsolminus = $nsol, $abcharge, $nsolminus\n";
	$line=sprintf("SOL%18d\n",$nsolminus);
	} # end if 
	print FILE2 $line;
} # end while

# now need to add new lines corresponding to NA+ or CL- ions
# NB currently no check if CL- or NA+ ions already present !
if ($charge>0) {
	$line=sprintf("CL-%18d\n",$abcharge);
	print FILE2 $line;
}
if ($charge < 0 ){
        $line=sprintf("NA+%18d\n",$abcharge);
	print FILE2 $line;
}
close(FILE1);
close(FILE2);
} # end sub modify_top_file



#------------------------------------
# Start of main program 
#------------------------------------

if (1+$#ARGV != 1) {
	&print_license_info();
        die "Usage: gromac_protein_dynamics.pl file.pdb\n";
}
$pdbfilename=shift(@ARGV);
print "PDB file to be used = $pdbfilename\n";
$lengthpdbname=length($pdbfilename);
$suffix=substr($pdbfilename,$lengthpdbname-4,4);
print "Suffix = $suffix\n";
if (($suffix ne '.pdb') && ($suffix ne '.PDB')) {
        print "PDB filename must have extension .pdb or .PDB \n";
        exit(0);
}
$prefix=substr($pdbfilename,0,$lengthpdbname-4);
$topfilename=$prefix.".top";
$grofilename=$prefix.".gro";
print "prefix = $prefix topfilename=$topfilename grofilename=$grofilename \n";

#
# ----------------------------------------
# pdb2gmx - Process input PDB file to produce
# (a) Gromacs coordinate (.gro) file.
# (b) Gromacs topology (.top) file
# ----------------------------------------
#
$command1="pdb2gmx -ff G43a1 -f $pdbfilename -o $grofilename -p $topfilename >& output.pdb2gmx";
&run_command($command1);

#
# ----------------------------------------
# editconf - define box size for system
# place solute in centre of box
# input and outout are
# Gromacs coordinate (.gro) file.
# ----------------------------------------
#
$boxsize=0.9;
$command2="editconf -bt cubic -f $grofilename -o $grofilename -c -d $boxsize >& output.editconf"; 
&run_command($command2);


#
# ----------------------------------------
# genbox - Add water molecules to system
# Generate gromacs coordinate file b4em (before energy minimisation)
# ----------------------------------------
#
$b4emgro="$prefix"."_b4em.gro";
$command3="genbox -cp $grofilename -cs spc216.gro -o $b4emgro -p $topfilename >& output.genbox";
&run_command($command3);


# (Q) can we run genion straight after genbox ?
# (A) No ! Must run grompp first to generate .tpr file
#     the .tpr file is a required input to genion

#
# ----------------------------------------
# create energy minisation parameter file
# needed for grompp
# ----------------------------------------
#
&create_em_mdp_file($prefix);

#
# ----------------------------------------
# Run grompp in order to
# preprocess files for energy minimisation
# The .tpr file produced can be used as input to genion
# ----------------------------------------
#
$b4emgro=$prefix."_b4em.gro";
$emtpr=$prefix."_em.tpr";
$command4="grompp -f em.mdp -c $b4emgro -p $topfilename -o $emtpr >& output.grompp_em";
&run_command($command4);
#$copy="cp $topfilename temp";
#system("$copy");

#
# ----------------------------------------
# set $grofile here
# may be changed to another name if genion is called
# ----------------------------------------
#
$grofile=$b4emgro;

#
# ----------------------------------------
# Find Total charge on system
# use command in back ticks to grab output from grep
# avoid running genion if total charge is zero !!
# ----------------------------------------
#
$charge_command="grep 'Total charge' output.pdb2gmx | awk '{print \$3}'";
print "charge command = $charge_command\n";
# used backticks rather than system command in order to return output
# from command to a scalar variable
$charge=`$charge_command`;
#
# need to strip carriage return off
#
chop($charge);
print "System total charge in PDB2GMX = $charge\n";

# ----------------------------------------
# only generate $command_genion and run genion
# if charge on system is non zero
# ----------------------------------------
if ($charge !=0) {
# Now reads total charge and hence number of ions from output.pdb2gmx
$iongro=$prefix."_ion.gro";
$ionlog=$prefix."_ion.log";
&create_sol_txt_file();
# if we get here then charge is non zero
# so set .gro file used to $iongro
# otherwise leave it as already set to $b4prgro
$grofile=$iongro;
if ($charge > 0) {
$nchloride=$charge;
$command_genion="genion -s $emtpr -o $grofile -nname Cl -nn $nchloride < sol.txt >& output.genion";
}
# CHECK - What if charge is 3.000003 etc ?
if ($charge <0) {
$nsodium=abs($charge);
$command_genion="genion -s $emtpr -o $grofile -pname Na -np $nsodium < sol.txt >& output.genion";
}
&run_command($command_genion);
} # end if non zero charge

# ----------------------------------------
# make substitutions in topology file
# ----------------------------------------
&modify_top_file($topfilename,$charge);



# ----------------------------------------
# run grompp to generate files for minimisation 
# ----------------------------------------
$command5="grompp -f em.mdp -c $grofile -p $topfilename -o $emtpr >& output.grompp_em";
&run_command($command5);


# ----------------------------------------
# run energy minimisation (Finally)
# ----------------------------------------
$b4pr=$grofile;
$emtrr=$prefix."_em.trr";
$command6="mdrun -nice 4 -s $emtpr -o $emtrr -c $grofile -v >& output.mdrun_em";
&run_command($command6);


# ---------------------------------------
# Create position restrained MD file (pr.mdp) 
# Run grompp to preprocess files - produces .tpr file     
# Run mdrun to perform position restrained MD
# ---------------------------------------
&create_pr_mdp_file($prefix,$charge);
$prtpr=$prefix."_pr.tpr";
$command7="grompp -f pr.mdp -c $grofile -p $topfilename -o $prtpr >& output.grompp_pr";
&run_command($command7);
$prtrr=$prefix."_pr.trr";
$command8="mdrun -nice 4 -s $prtpr -o $prtrr -c $grofile -v >& output.mdrun_pr";
&run_command($command8);

# ----------------------------------------
# Create MDP file for unrestrained MD (md.mdp)
# Run grompp to preprocess files and producr .tpr file
# Run mdrun to perform unrestrained MD - produces .trr file
# ----------------------------------------
&create_md_mdp_file($prefix,$charge);
$mdtpr=$prefix."_md.tpr";
$command9="grompp -f md.mdp -c $grofile -p $topfilename -o $mdtpr >& output.grompp_md";
&run_command($command9);
$mdtrr=$prefix."_md.trr";
$command10="mdrun -nice 4 -s $mdtpr -o $mdtrr -c $grofile -v >& output.mdrun_md";
&run_command($command10);


# ----------------------------------------
# pull out any errors in output files
# ----------------------------------------
system("grep -i error output.*");
exit(0);


# ----------------------------------------
# end of script
# ----------------------------------------




